#include "../stdafx.h"

#include "MyDemo.h"
#include "Scenario.h"

#include "../engine/camera/Camera.h"
#include "../engine/TextRenderer.h"
//#include "../engine/Billboarder.h"
#include "../engine/AppContext.h"
#include "../utils/Config.h"
#include "../utils/DXHelpers.h"

#include "objs/WhiteLines.h"
#include "objs/RedBlob.h"
#include "objs/Wheel.h"
#include "objs/RedQuad.h"
#include "objs/GrayLines.h"
#include "RTMesh.h"

con::CChannel& CMyDemo::CONSOLE = con::CConsole::getChannel( "demo" );


// ------------------------------------------------------------------
//  basic stuff


CMyDemo::CMyDemo( CConfig& config )
:	CBaseDemo( config ),
	mCamera( NULL ),
	mTextRenderer( NULL ),
	//mBillboarder( NULL ),
	mEntities( NULL )//,
	//mCrapIsSet( false ),
	//mCrapShouldBe( false )
{
	for( int i = 0; i < RT_COUNT; ++i ) {
		mRTTex[i] = NULL;
		mRTTexCopy[i] = NULL;
		mRTDepth[i] = NULL;
		mRT[i] = NULL;
	}
}

CMyDemo::~CMyDemo()
{
}


// ------------------------------------------------------------------
//  initialization


HRESULT CMyDemo::onResetDevice()
{
	CBaseDemo::onResetDevice();

	IDirect3DSurface8* rt;
	getDevice().GetRenderTarget( &rt );
	assert( rt );
	D3DSURFACE_DESC desc;
	rt->GetDesc( &desc );

	int size = 128;
	for( int i = 0; i < RT_COUNT; ++i ) {
		getDevice().CreateTexture( size, size, 1, D3DUSAGE_RENDERTARGET, desc.Format, D3DPOOL_DEFAULT, &mRTTex[i] );
		assert( mRTTex[i] );
		getDevice().CreateTexture( size, size, 1, 0, desc.Format, D3DPOOL_DEFAULT, &mRTTexCopy[i] );
		assert( mRTTexCopy[i] );
		getDevice().CreateDepthStencilSurface( size, size, D3DFMT_D16, D3DMULTISAMPLE_NONE, &mRTDepth[i] );
		assert( mRTDepth[i] );
		mRT[i] = CRenderTarget::createRenderTarget(
			*mRTTex[i], mRTDepth[i], false, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, 0xFF000000 );
		size /= 2;
	}

	rt->Release();

	return S_OK;
}

HRESULT CMyDemo::onLostDevice()
{
	CBaseDemo::onLostDevice();

	for( int i = 0; i < RT_COUNT; ++i ) {
		dx::release( mRTTex[i] );
		dx::release( mRTTexCopy[i] );
		dx::release( mRTDepth[i] );
		if( mRT[i] )
			delete mRT[i];
	}
	
	return S_OK;
}


void CMyDemo::onInitialize()
{
	// camera
	mCamera = new CCamera();

	// entities
	mEntities = new CDemoEntityContainer();

	// text renderer
	std::string fontName = getConfig().readS( "screen", "fontName", "Arial" );
	int fontSize = getConfig().readI( "screen", "fontSize", 9 );
	bool fontBold = getConfig().readI( "screen", "fontBold", 1 ) ? true : false;
	bool fontItalic = getConfig().readI( "screen", "fontItalic", 0 ) ? true : false;

	mTextRenderer =  new CTextRenderer( fontName.c_str(), fontSize, fontBold, fontItalic );
	getObjects().addObject( *mTextRenderer );

	// billboarder
	//mBillboarder = new CBillboarder( *mCamera );
	//getObjects().addObject( *mBillboarder );


	//
	// init demo objects


	CRedQuadEntity* redQuad = new CRedQuadEntity();
	mEntities->addEntity( *redQuad );

	CRedBlobEntity* redBlob = new CRedBlobEntity();
	mEntities->addEntity( *redBlob );
	
	CWhiteLinesEntity* whiteLines = new CWhiteLinesEntity( *mCamera );
	mEntities->addEntity( *whiteLines );

	CWheelEntity* wheel = new CWheelEntity( *mCamera );
	mEntities->addEntity( *wheel );

	CGrayLinesEntity* grayLines = new CGrayLinesEntity( *mCamera );
	mEntities->addEntity( *grayLines );

	//
	// meshes

	mCredits = new CMesh( CResourceId("billboard.x"), CResourceId("Credits.sha"), CResourceId("credits.png") );
	getObjects().addObject( *mCredits );
	mCredits->setVisible( false );

	/*
	for( int i = 0; i < CRAP_COUNT; ++i ) {
		std::string texname = "crap";
		texname += ('1'+i);
		texname += ".png";
		CResourceId tex( texname );
		mCrapMeshes[i] = new CMesh( CResourceId("billboard.x"), CResourceId("Credits.sha"), tex );
		getObjects().addObject( *mCrapMeshes[i] );
		mCrapMeshes[i]->setVisible( false );
	}
	*/
	
	//
	// cubelet

	/*
	mCubelet = new CMesh( CResourceId("Kubukas.x"), CResourceId("EnvMesh.sha"), CResourceId("specular.png") );
	getObjects().addObject( *mCubelet );
	mCubelet->setVisible( false );
	D3DXMATRIX& m = mCubelet->getTextureMatrix();
	D3DXMatrixIdentity( &m );
	m._11 = 0.5f;
	m._22 = -0.5f;
	m._41 = 0.5f;
	m._42 = 0.5f;
	*/


	//
	// RTs

	mRTMesh0 = new CRTMesh( CResourceId("Billboard.x"), CResourceId("AdditiveMesh.sha") );
	getObjects().addObject( *mRTMesh0 );
	mRTMesh1 = new CRTMesh( CResourceId("Billboard.x"), CResourceId("AdditiveMesh.sha") );
	getObjects().addObject( *mRTMesh1 );


	//
	// init scenario

	CScenario::getInstance().init( getAppContext().getTime().getTime() );
	mEntities->onSceneStart( CScenario::getInstance().getCurrScene() );
}


// ------------------------------------------------------------------
//  game logic


void CMyDemo::onUpdate()
{
	// roll scenario
	float time = getAppContext().getTime().getTime();
	CScenario& scenario = CScenario::getInstance();
	CScenario::eScene oldScene = scenario.getCurrScene();
	scenario.update( time );
	CScenario::eScene newScene = scenario.getCurrScene();
	if( oldScene != newScene ) {
		CONSOLE << "switch scene from " << oldScene << " to " << newScene << con::endl;
		mEntities->onSceneEnd( oldScene );
		mEntities->onSceneStart( newScene );
	}
	mEntities->update();
}


// ------------------------------------------------------------------
//  rendering

void CMyDemo::onRenderBeforeAll()
{
	// roll scenario - needed at rendering also
	float time = getAppContext().getTime().getTime();
	CScenario& scenario = CScenario::getInstance();
	CScenario::eScene oldScene = scenario.getCurrScene();
	scenario.update( time );
	CScenario::eScene newScene = scenario.getCurrScene();
	if( oldScene != newScene ) {
		CONSOLE << "switch scene from " << oldScene << " to " << newScene << con::endl;
		mEntities->onSceneEnd( oldScene );
		mEntities->onSceneStart( newScene );
	}

	// clear
	DWORD clearColor = 0xFF000000;

	getDevice().Clear( 0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, clearColor, 1.0f, 0 );

	// RTT params

	setRTParams();

	mCamera->setProjectionParams(
		mCamera->getFOV(),
		getAppContext().getWindowAspect(),
		mCamera->getZNear(),
		mCamera->getZFar()
	);
	mCamera->setDeviceTransforms( getDevice(), true, true );

	// before render
	mEntities->beforeRendering();

	// RTTs
	int i;

	for( i = 0; i < RT_COUNT; ++i ) {
		RECT rc;
		rc.top = 0; rc.left = 0; rc.right = (128 >> i); rc.bottom = (128 >> i);
		POINT pt;
		pt.x = 0; pt.y = 0;

		IDirect3DSurface8* src;
		mRTTex[i]->GetSurfaceLevel( 0, &src );
		IDirect3DSurface8* dst;
		mRTTexCopy[i]->GetSurfaceLevel( 0, &dst );

		getDevice().CopyRects( src, &rc, 1, dst, &pt );

		src->Release();
		dst->Release();
	}

	mRTMesh0->setVisible( false );

	mRTMesh1->setVisible( true );
	for( i = 0; i < RT_COUNT; ++i ) {
		mRTMesh1->getWorldMatrix()._41 = -20.0f / (128 >> i);
		mRTMesh1->getWorldMatrix()._42 = 20.0f / (128 >> i);
		mRTMesh1->setRTT( *mRTTex[i] );
		mRT[i]->setClearBeforeUse( true );
		mRT[i]->apply();
		getObjects().render( getAppContext().getTime() );
		//renderCubelets();
		mRT[i]->restore();
	}

	mRTMesh1->setVisible( false );
}


void CMyDemo::setRTParams()
{
	float t = CScenario::getInstance().getCurrentTime();

	mRTMesh0->getColor() = mRTMesh1->getColor() = 0xFF808080;
	//mRTMesh0->getColor() = 0xFFF0F0F0;
	//mRTMesh1->getColor() = 0xFFF0F0F0;

	// correct size
	D3DXMATRIX& m = mRTMesh0->getWorldMatrix();
	D3DXMatrixIdentity( &m );
	m._11 = 40.0f * getAppContext().getWindowAspect();
	m._22 = 40.0f;
	m._43 = 20.0f;

	// zoom?
	const float zoomperiod = 8.0f;
	const float zoomdur = 3.0f;
	const float zoommod = fmodf( t, zoomperiod );
	if( zoommod < zoomdur ) {
		mRTMesh0->getColor() = mRTMesh1->getColor() = 0xFFC0C0C0;
		const float alpha = zoommod/zoomdur;
		m._43 -= sinf( alpha * 3.1415f ) * 2.0f;
	}

	// rotate?
	const float rotperiod = 11.0f;
	const float rotdur = 3.0f;
	const float rotmod = fmodf( t, rotperiod );
	if( rotmod < rotdur ) {
		const float alpha = rotmod/rotdur;
		D3DXMATRIX mrot;
		D3DXMatrixRotationZ( &mrot, sinf( alpha * 2.0f * 3.1415f ) * 0.3f );
		D3DXMatrixMultiply( &m, &mrot, &m );
	}

	// strobe?
	const float strobeperiod = 0.5f;
	const float strobedur = 0.05f;
	const float strobemod = fmodf( t, strobeperiod );
	if( strobemod < strobedur ) {
		mRTMesh0->getColor() = mRTMesh1->getColor() = 0xFFFFFFFF;
		const float alpha = strobemod/strobedur;
		const float f = sinf( alpha * 3.1415926f );
		mCamera->getCameraMatrix()._43 = f * 1.0f;
	} else {
		mCamera->getCameraMatrix()._43 = 0.0f;
	}
	mCamera->updateMatrices();


	mRTMesh1->getWorldMatrix() = m;
}

void CMyDemo::onRenderAfterAll()
{
	//renderCubelets();
	int i;

	mRTMesh0->setVisible(true);
	for( i = 0; i < RT_COUNT; ++i ) {
		mRTMesh0->getWorldMatrix()._41 = -20.0f / (128 >> i);
		mRTMesh0->getWorldMatrix()._42 = 20.0f / (128 >> i);
		mRTMesh0->setRTT( *mRTTexCopy[i] );
		mRTMesh0->render( getAppContext().getTime() );
	}
	mRTMesh0->setVisible(false);


	// restore camera
	mCamera->getCameraMatrix()._43 = 0.0f;
	mCamera->updateMatrices();
	mCamera->setDeviceTransforms( getDevice(), true, false );

	//
	// crap

	/*
	const float strobeperiod = 1.5f;
	const float strobedur = 0.2f;
	float t = CScenario::getInstance().getCurrentTime();
	const float strobemod = fmodf( t, strobeperiod );
	if( strobemod < strobedur ) {
		if( !mCrapIsSet ) {
			mCrapShouldBe = (rand()&15) > 10;
			mCrapNumber = rand()%CRAP_COUNT;
		}
		mCrapIsSet = true;

		if( mCrapShouldBe ) {
			float alpha = strobemod / strobeperiod;
			float f = sinf( alpha * 3.1415926f );

			D3DXCOLOR color = 0xFF000000;
			color.r = color.g = color.b = f * 0.5f;
			mCrapMeshes[mCrapNumber]->getColor() = color;
			
			mCrapMeshes[mCrapNumber]->setVisible(true);
			D3DXMATRIX& m = mCrapMeshes[mCrapNumber]->getWorldMatrix();
			m._11 = 40.0f * getAppContext().getWindowAspect();
			m._22 = 40.0f;
			m._43 = 20.0f;
			mCrapMeshes[mCrapNumber]->render( getAppContext().getTime() );
			mCrapMeshes[mCrapNumber]->setVisible(false);
		}
	} else {
		mCrapIsSet = false;
		mCrapShouldBe = false;
	}

  */
	//
	// credits

	//if( CScenario::getInstance().getCurrScene() == CScenario::END ) {
		mCredits->setVisible(true);
		D3DXMATRIX& m = mCredits->getWorldMatrix();
		m._11 = 16.0f;
		m._22 = m._11 * 0.5f;
		m._41 = 20.0f * getAppContext().getWindowAspect() - m._11*0.5f;
		m._42 = -20.0f + m._22*0.4f;
		m._43 = 20.0f;
		mCredits->render( getAppContext().getTime() );
		mCredits->setVisible(false);
	//}

	mTextRenderer->renderScreenText( 5, 5, getAppContext().getFPS(), 0x80800000 );

	if( CScenario::getInstance().getCurrScene() == CScenario::MAX_SCENES )
		setEnded( true );
}


/*
void CMyDemo::renderCubelets()
{
	float t = CScenario::getInstance().getCurrentTime();

	// cubelets?
	const float cubeperiod = 5.0f;
	const float cubedur = 3.0f;
	const float cubemod = fmodf( t-cubedur*1.1f, cubeperiod );
	if( cubemod < cubedur ) {
		const float alpha = cubemod/cubedur;
		mCubelet->setVisible(true);

		const int CUBES = 50;
		for( int i = 0; i < CUBES; ++i ) {
			D3DXMATRIX& m = mCubelet->getWorldMatrix();
			D3DXMatrixIdentity( &m );
			m._11 = m._22 = m._33 = 4.0f;

			D3DXMATRIX mrot;
			D3DXMatrixRotationYawPitchRoll( &mrot, cosf(alpha*8.3f+i*0.13f), cosf(alpha*17.9f+i*0.31f), cosf(alpha*11.1f+i*0.19f) );
			D3DXMatrixMultiply( &m, &mrot, &m );

			m._41 = (cosf( i ) + sinf( i - 3 )) * 20.0f;
			m._42 = (sinf( i+1 ) + sinf( i * 0.71f + 0.5f )) * 20.0f;
			m._43 = 200.0f - alpha * 200.0f + cosf(i*0.21f)*40.0f;
			mCubelet->render( getAppContext().getTime() );
		}

		mCubelet->setVisible(false);
	}
}
*/