/**
 *  @file game/MyDemo.h
 *  Your demo application code.
 */

#ifndef __MY_DEMO_H
#define __MY_DEMO_H

#include "DemoEntity.h"
#include "../engine/BaseDemo.h"
#include "../utils/Singleton.h"
#include "../common/console/Console.h"
#include "../utils/Perlin.h"
#include "../engine/RenderTarget.h"

class CCamera;
class CTextRenderer;
class CBillboarder;
class CRTMesh;
class CMesh;


/**
 *  Your demo application class.
 */
class CMyDemo : public CBaseDemo, public CSingleton<CMyDemo> {
public:
	static con::CChannel& CONSOLE;
	
	static void init( CConfig& config ) {
		CMyDemo* game = new CMyDemo( config ); assert( game );
		assignInstance( *game );
	}
	virtual ~CMyDemo();

public:
	const CPerlin& getPerlin() const { return mPerlin; }

	// CDXObject
	virtual HRESULT onLostDevice();
	virtual HRESULT onResetDevice();

protected:
	/**
	 *  Initialize the demo.
	 */
	virtual	void onInitialize();
	
	/**
	 *  Execute demo logic.
	 *  The engine calls this method when update time comes.
	 */
	virtual void onUpdate();

	/**
	 *  Perform additional rendering.
	 *  The engine calls this method before rendering anything.
	 */
	virtual void onRenderBeforeAll();

	/**
	 *  Perform additional rendering.
	 *  The engine calls this method after rendering everything.
	 */
	virtual void onRenderAfterAll();


private:
	CMyDemo( CConfig& config );

	static CMyDemo* createInstance() {
		assert( !"Must be initialized first" );
		return 0;
	}

	static void deleteInstance( CMyDemo& instance ) {
		delete &instance;
	}

	friend CSingleton<CMyDemo>;	

private:
	void	setRTParams();
	//void	renderCubelets();
	
private:
	CCamera*		mCamera;
	CTextRenderer*	mTextRenderer;
	//CBillboarder*	mBillboarder;
	CDemoEntityContainer*	mEntities;
	CPerlin			mPerlin;

	enum { RT_COUNT = 4 };
	IDirect3DTexture8*	mRTTex[RT_COUNT];
	IDirect3DTexture8*	mRTTexCopy[RT_COUNT];
	IDirect3DSurface8*	mRTDepth[RT_COUNT];
	CRenderTarget*		mRT[RT_COUNT];
	CRTMesh*			mRTMesh0;
	CRTMesh*			mRTMesh1;
	CMesh*				mCredits;

	/*
	enum { CRAP_COUNT = 3 };
	CMesh*				mCrapMeshes[CRAP_COUNT];
	bool				mCrapIsSet;
	bool				mCrapShouldBe;
	int					mCrapNumber;
	*/

	//CMesh*				mCubelet;
};

#endif
