
#ifndef __SCENARIO_H
#define __SCENARIO_H

#include "../utils/Singleton.h"


class CScenario : public CSingleton<CScenario> {
public:
	enum eScene {
		WHITELINE = 0,
		LINES,
		LINES_BLOB,
		LINES_CIRCLE_BLOB,
		CIRCLE_BLOB,
		WHEEL_BLOB,
		WHEEL_QUAD_BLOB,
		WHEEL_QUAD_BLOB_LINES,
		FADEWHEEL_QUAD_BLOB_LINES,
		FADE,
		END,
		MAX_SCENES
	};

public:
	float	getSceneDuration( eScene scene ) const { return mSceneDurations[scene]; }
	float	getSceneStart( eScene scene ) const { return mSceneStarts[scene]; }

	float	getCurrentTime() const { return mCurrentTime; }
	eScene	getCurrScene() const { return mCurrScene; }
	float	getCurrSceneTime() const { return mCurrSceneTime; }
	float	getCurrSceneLeft() const { return mCurrSceneLeft; }
	float	getCurrSceneAlpha() const { return mCurrSceneAlpha; }

	void	init( float time ) { mInitTime = time; mCurrScene = (eScene)0; }
	void	update( float time );

private:
	CScenario();
	static CScenario* createInstance() { return new CScenario(); }
	static void deleteInstance( CScenario& instance ) { delete &instance; }
	friend CSingleton<CScenario>;

private:
	float	mSceneDurations[MAX_SCENES+1];
	float	mSceneStarts[MAX_SCENES+1];


	float	mInitTime;
	float	mCurrentTime;
	eScene	mCurrScene;
	float	mCurrSceneTime;
	float	mCurrSceneLeft;
	float	mCurrSceneAlpha;
};


#endif
