#include "../../stdafx.h"

#include "RedBlob.h"
#include "../MyDemo.h"
#include "../../utils/DXHelpers.h"
#include "../../common/resource/TextureBundle.h"
#include "../../common/resource/EffectBundle.h"


// ------------------------------------------------------------------
//  basic stuff


CRedBlobEntity::CRedBlobEntity()
:	mMesh( NULL )
{
	CMyDemo	::getInstance().getObjects().addObject( *this );
	setVisible( false );
}

CRedBlobEntity::~CRedBlobEntity()
{
	CMyDemo::getInstance().getObjects().removeObject( *this );
}


// ------------------------------------------------------------------
//  DX part

HRESULT CRedBlobEntity::onLostDevice()
{
	CDXObject::onLostDevice();
	delete[] mOriginal;
	dx::release( mMesh );
	return S_OK;
}

const float RADIUS = 60.0f;
const int STEPS = 20;

HRESULT CRedBlobEntity::onResetDevice()
{
	CDXObject::onResetDevice();

	// create sphere
	ID3DXMesh* mesh;
	ID3DXBuffer* adj;
	D3DXCreateSphere( &getDevice(), RADIUS, STEPS, STEPS, &mesh, &adj );
	mesh->OptimizeInplace( D3DXMESHOPT_COMPACT | D3DXMESHOPT_ATTRSORT | D3DXMESHOPT_VERTEXCACHE, (const DWORD*)adj->GetBufferPointer(), NULL, NULL, NULL );

	// read original vertices
	mVertexCount = mesh->GetNumVertices();
	
	mOriginal = new D3DXVECTOR3[ mVertexCount ];
	int vsize = D3DXGetFVFVertexSize( mesh->GetFVF() );
	const BYTE* p;
	mesh->LockVertexBuffer( D3DLOCK_READONLY, (BYTE**)&p );
	for( int i = 0; i < mVertexCount; ++i ) {
		mOriginal[i] = *(const D3DXVECTOR3*)( p + i*vsize );
	}
	mesh->UnlockVertexBuffer();
	
	// clone mesh to our dynamic mesh
	mesh->CloneMeshFVF( D3DXMESH_DYNAMIC | D3DXMESH_WRITEONLY, MY_FVF, &getDevice(), &mMesh );

	// release stuff
	adj->Release();
	mesh->Release();
	
	return S_OK;
}



// ------------------------------------------------------------------
//  Rendering

void CRedBlobEntity::render( CFrameTime const& frameTime )
{
	if( !isVisible() )
		return;

	// draw
	ID3DXEffect& effect = *CEffectBundle::getInstance().getResourceById( CResourceId("RedBlob.sha") );
	IDirect3DTexture8& texture = *CTextureBundle::getInstance().getResourceById( CResourceId("lineHard.png") );
	effect.SetTexture( "texBase", &texture );
	UINT passes;
	effect.Begin( &passes, 0 );

	for( UINT p = 0; p < passes; ++p ) {
		effect.Pass( p );
		mMesh->DrawSubset(0);
	}
	effect.End();
}

void CRedBlobEntity::beforeRendering()
{
	if( !isVisible() )
		return;
	
	const CScenario& scenario = CScenario::getInstance();

	// manage vertices
	float t = scenario.getCurrentTime() * 2.5f;
	float tx = t;
	float ty = t * 1.3f;
	float tz = t * 1.7f;
	float m = 2.0f / RADIUS;


	float k1 = powf(cosf(tz-cosf(tx)),10.0f) * 0.4f + 0.2f;

	float dist = 40.0f;
	if( scenario.getCurrScene() == CScenario::LINES_BLOB ) {
		dist += (1-scenario.getCurrSceneAlpha()) * 100.0f;
	} else if( scenario.getCurrScene() == CScenario::FADE ) {
		dist += scenario.getCurrSceneAlpha() * 50.0f;
	}

	TMyVertex *vb;
	mMesh->LockVertexBuffer( D3DLOCK_DISCARD, (BYTE**)&vb );

	for( int i = 0; i < mVertexCount; ++i ) {
		D3DXVECTOR3 orig = mOriginal[i];
		
		float s = 1.0f + cosf( orig.x * m + tx + cosf( orig.y * m + ty + cosf( orig.z * m + tz ) ) ) * k1;
		D3DXVECTOR3 pos = orig * s;

		vb->p.x = pos.x;
		vb->p.y = pos.y;
		vb->p.z = pos.z * 0.2f + dist;
		D3DXCOLOR color = 0xFF000000;
		color.r = 0.35f - (pos.x*pos.x+pos.y*pos.y)*0.001f;
		color.r += cosf( pos.z * 0.2f + pos.y*0.061f - pos.x * 0.07f + t ) * 0.2f;
		if( s > 1.3f ) {
			//color.g = sinf( s-1.3f );
			//color.b = sinf( (s-1.3f) * 0.7f );
			color.g = color.r * 0.3f;
		}
		vb->diffuse = color;
		vb->tu = pos.x * 0.1f;
		vb->tv = pos.y * 0.1f;
		
		vb++;
	}

	mMesh->UnlockVertexBuffer();


	// draw
	ID3DXEffect& effect = *CEffectBundle::getInstance().getResourceById( CResourceId("RedBlob.sha") );
	IDirect3DTexture8& texture = *CTextureBundle::getInstance().getResourceById( CResourceId("lineHard.png") );
	effect.SetTexture( "texBase", &texture );
	UINT passes;
	effect.Begin( &passes, 0 );

	for( UINT p = 0; p < passes; ++p ) {
		effect.Pass( p );
		mMesh->DrawSubset(0);
	}
	effect.End();
}

bool CRedBlobEntity::isActive( CScenario::eScene scene ) const
{
	return scene >= CScenario::LINES_BLOB && scene <= CScenario::FADE;
}

void CRedBlobEntity::onSceneStart( CScenario::eScene scene )
{
	if( scene == CScenario::LINES_BLOB ) {
		setVisible( true );
	}
}

void CRedBlobEntity::onSceneEnd( CScenario::eScene scene )
{
	if( scene == CScenario::FADE ) {
		setVisible(false);
	}
}

void CRedBlobEntity::update()
{
}

