#include "../../stdafx.h"

#include "RedQuad.h"
#include "../MyDemo.h"
#include "../../common/resource/ResourceId.h"



CRedQuadEntity::CRedQuadEntity()
:	mMesh( NULL )
{
	mMesh = new CMesh( CResourceId("billboard.x"), CResourceId("TexturedMesh.sha"), CResourceId("dummy.png") );
	
	CMyDemo::getInstance().getObjects().addObject( *mMesh );
	mMesh->setVisible( false );
}

CRedQuadEntity::~CRedQuadEntity()
{
	CMyDemo::getInstance().getObjects().removeObject( *mMesh );
}

bool CRedQuadEntity::isActive( CScenario::eScene scene ) const
{
	return scene >= CScenario::WHEEL_QUAD_BLOB && scene <= CScenario::END;
}

void CRedQuadEntity::onSceneStart( CScenario::eScene scene )
{
	if( scene == CScenario::WHEEL_QUAD_BLOB ) {
		mMesh->setVisible( true );
		mMesh->getColor() = 0xFFC00000;
	}
}

void CRedQuadEntity::onSceneEnd( CScenario::eScene scene )
{
	if( scene == CScenario::END ) {
		mMesh->setVisible( false );
	}
}

void CRedQuadEntity::beforeRendering()
{
	const CScenario& scenario = CScenario::getInstance();
	CScenario::eScene scene = scenario.getCurrScene();

	if( scene == CScenario::END ) {
		float alpha = scenario.getCurrSceneAlpha();
		
		D3DXMATRIX& m = mMesh->getWorldMatrix();
		float scale = 50.0f;
		D3DXMatrixScaling( &m, scale, scale, scale );
		m._43 = 15.0f;

		D3DXCOLOR color = 0xFF000000;
		color.r = alpha * 0.8f;
		mMesh->getColor() = color;
	} else {
		float timeStart = scenario.getSceneStart( CScenario::WHEEL_QUAD_BLOB );
		float timeEnd = scenario.getSceneStart( CScenario::END );
		float alpha = (scenario.getCurrentTime()-timeStart) / (timeEnd-timeStart);
		
		D3DXMATRIX& m = mMesh->getWorldMatrix();

		// rotate
		float rot = -alpha*30.0f + cosf( alpha * 9.1f + sinf( alpha * 13.9f ) * 3.0f ) * 3.0f - sinf( (alpha-0.3f) * 16.9f ) * 2.0f;
		D3DXMatrixRotationZ( &m, rot );

		// scale
		D3DXMATRIX mscale;
		float scale = 48.0f - 45.0f*alpha;
		D3DXMatrixScaling( &mscale, scale, scale, scale );
		D3DXMatrixMultiply( &m, &m, &mscale );

		if( scene == CScenario::FADE ) {
			float alpha = powf( scenario.getCurrSceneAlpha(), 3.0f );
			D3DXMATRIX& m = mMesh->getWorldMatrix();
			m._41 = -alpha * 80.0f;
			m._42 = alpha * 30.0f;
			m._43 = 15.0f + alpha * 10.0f;
		} else {
			m._43 = 15.0f;
		}
	}
}

