#include "../stdafx.h"

#include "BaseDemo.h"
#include "AppContext.h"
#include "FrameTime.h"

#include "../utils/Config.h"

#include "Constants.h"


// ------------------------------------------------------------------
//  basic stuff

CBaseDemo::CBaseDemo( CConfig& config )
:	mConfig( config ),
	mAppContext( NULL ),
	mTimeSinceLastActualUpdate( 0 ),
	mEnded(false)
{
}

CBaseDemo::~CBaseDemo()
{
}


// ------------------------------------------------------------------
//  CDXObjectContainer

HRESULT CBaseDemo::onCreateDevice( IDirect3DDevice8* device )
{
	CDXObject::onCreateDevice( device );
	static bool once = true;
	if( once ) {
		// demo init
		onInitialize();
		once = true;
	}
	return mObjects.onCreateDevice( device );
}

HRESULT CBaseDemo::onResetDevice()
{
	assert( mAppContext );

	CDXObject::onResetDevice();

	return mObjects.onResetDevice();
}

HRESULT CBaseDemo::onLostDevice()
{
	CDXObject::onLostDevice();
	return mObjects.onLostDevice();
}

HRESULT CBaseDemo::onDestroyDevice()
{
	CDXObject::onLostDevice();
	return mObjects.onDestroyDevice();
}

void CBaseDemo::render( CFrameTime const& frameTime )
{
	assert( mAppContext );

	float alpha = mTimeSinceLastActualUpdate / gcon::UPDATE_DT;
	getAppContext().getTime().setAlpha( alpha );

	// game: before
	onRenderBeforeAll();

	// render
	mObjects.render( frameTime );

	// game: after
	onRenderAfterAll();
}

void CBaseDemo::update()
{
	float t = getAppContext().getTime().getDelta() + mTimeSinceLastActualUpdate;
	while( t >= gcon::UPDATE_DT ) {
		onUpdate();
		t -= gcon::UPDATE_DT;

		mTimeSinceLastActualUpdate = 0;
	}

	mTimeSinceLastActualUpdate = t;
}

