#ifndef __BASE_DEMO_H
#define __BASE_DEMO_H

#include "DXObject.h"
#include "FrameTime.h"

class CConfig;
class CAppContext;


/**
 *  Base demo class.
 */
class CBaseDemo : public CDXObject
{
public:
	explicit CBaseDemo( CConfig& config );
	virtual ~CBaseDemo();

	/// Returns read-only app context.
	const CAppContext& getAppContext() const { return *mAppContext; }
	/// Returns app context.
	CAppContext& getAppContext() { return *mAppContext; }

	/// Returns read-only config.
	const CConfig& getConfig() const { return mConfig; }
	/// Returns config.
	CConfig& getConfig() { return mConfig; }

	/// Returns read-only visible objects container.
	const CDXObjectContainer& getObjects() const { return mObjects; }
	/// Returns visible objects container.
	CDXObjectContainer& getObjects() { return mObjects; }

	//CFrameTime const& getHackFrameTimeForAnimation() { return mHackFrameTimeForAnimation; }


	// CDXObject
	virtual HRESULT onCreateDevice( IDirect3DDevice8* device );
	virtual HRESULT onLostDevice();
	virtual HRESULT onResetDevice();
	virtual HRESULT onDestroyDevice();
	virtual void render( CFrameTime const& frameTime );

	// interface methods for CGameApp class instance
	void update();
	void setAppContext( CAppContext& appContext ) { mAppContext = &appContext; }
	bool isEnded() const { return mEnded; }

protected:
	/**
	 *  Initialize the demo.
	 */
	virtual void	onInitialize() { };
	
	/**
	 *  Execute logic.
	 *  The engine calls this method when update time comes.
	 */
	virtual void	onUpdate() { };

	/**
	 *  Perform additional rendering.
	 *  The engine calls this method before rendering anything.
	 */
	virtual void	onRenderBeforeAll() { };

	/**
	 *  Perform additional rendering.
	 *  The engine calls this method after rendering everything.
	 */
	virtual void	onRenderAfterAll() { };

	void setEnded( bool ended ) { mEnded = ended; }

private:
	CConfig&			mConfig;

	CAppContext*		mAppContext;

	CDXObjectContainer	mObjects;

	float				mTimeSinceLastActualUpdate;

	bool				mEnded;
};

#endif
