#include "../stdafx.h"

#include "DemoApp.h"
#include "Constants.h"

#include "../common/resource/MeshBundle.h"
#include "../common/resource/EffectBundle.h"
#include "../common/resource/TextureBundle.h"
#include "../common/console/Console.h"
#include "../common/console/W32StdConsoleRenderingContext.h"
#include "../common/console/WDebugConsoleRenderingContext.h"

#include "BaseDemo.h"

//#include "camera/Camera.h"

#include "../utils/Config.h"


CDemoApp::CDemoApp( CBaseDemo& demo, CConfig& config )
:	mDemo( demo ),
	mConfig( config ),
	mFrameTime( 0, 0, 0 ),
	mAppContext( mFrameTime )
{
	//
	// set up app

	std::string title = mConfig.readS( "screen", "title", "Demo" );
	m_strWindowTitle    = const_cast<char*>( title.c_str() );
	m_dwMinDepthBits	= 16;
	m_dwMinStencilBits	= 0;
	m_dwCreationWidth	= mConfig.readI( "screen", "width", 640 );
	m_dwCreationHeight	= mConfig.readI( "screen", "height", 480 );
    m_bUseDepthBuffer   = TRUE;

	mWindowed = mConfig.readI( "screen", "windowed", 1 ) ? true : false;

	mDemo.setAppContext( mAppContext );
}

CDemoApp::~CDemoApp()
{
}

HRESULT CDemoApp::OneTimeSceneInit()
{
	//con::IConsoleRenderingContext* ctx = new CW32StdConsoleRenderingContext( m_hWnd );
	con::IConsoleRenderingContext* ctx = new CWDebugConsoleRenderingContext();
	con::CConsole::getConsole()->setDefaultRenderingContext( *ctx );

    return S_OK;
}

HRESULT CDemoApp::InitDeviceObjects()
{
	assert( m_pd3dDevice );

	/*
	bool windowed = mConfig.readI( "screen", "windowed", 1 ) ? true : false;
	if( !windowed )
		ForceFullscreen();
		*/

	CMeshBundle::init( *m_pd3dDevice, "mesh/" );
	CEffectBundle::init( *m_pd3dDevice, "efx/" );
	CTextureBundle::init( *m_pd3dDevice, "tex/" );
	
	mDemo.onCreateDevice( m_pd3dDevice );

    return S_OK;
}

HRESULT CDemoApp::RestoreDeviceObjects()
{
	mDemo.onResetDevice();

	return S_OK;
}

HRESULT CDemoApp::InvalidateDeviceObjects()
{
	CEffectBundle::getInstance().clear();
	mDemo.onLostDevice();

	return S_OK;
}

HRESULT CDemoApp::DeleteDeviceObjects()
{
	CTextureBundle::getInstance().clear();
	CMeshBundle::getInstance().clear();

	mDemo.onResetDevice();

	return S_OK;
}

HRESULT CDemoApp::Render()
{
	if( mDemo.isEnded() )
		return S_OK;
	
	if( !mWindowed && m_bWindowed )
		ToggleFullscreen();

	HRESULT hr;

	if( FAILED( hr = m_pd3dDevice->BeginScene() ) )
		return hr;

	mDemo.render( mFrameTime );
	
	if( FAILED( hr = m_pd3dDevice->EndScene() ) )
		return hr;

	return S_OK;
}

HRESULT CDemoApp::FrameMove()
{
	// frame time
	int frameNr = mFrameTime.getFrameNumber();
	
	float elapsedTime = m_fElapsedTime;
	if( frameNr == 0 )
		elapsedTime = gcon::UPDATE_DT;

	mFrameTime = CFrameTime( m_fTime, elapsedTime, frameNr + 1 );

	// app context
	mAppContext.setInfo(
		(float)m_d3dsdBackBuffer.Width / (float)m_d3dsdBackBuffer.Height,
		std::string( m_strFrameStats ),
		std::string( m_strDeviceStats ) );

	mDemo.update();

	if( mDemo.isEnded() ) {
		m_bActive = false;
		m_bReady = false;
		::SendMessage( m_hWnd, WM_CLOSE, 0, 0 );
	}

	return S_OK;
}


LRESULT CDemoApp::MsgProc( HWND wnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	return CD3DApplication::MsgProc( wnd, msg, wParam, lParam );
}
